/*
keyFinder.js
Version 1.01

A JavaScript tool that helps navigate the Java API documentation.

Copyright 2002 by Steve Claflin

Currently freeware--licensed freely for use on your own system, but not for distribution to others.
*/
/* Revision History

   Ver 1.01 - Feb 14, 2002 
     Added Netscape 6 compatibility
     Started using constants for named keys (like CR = 13)
     Revised IE goTo code for efficiency by 
        storing e.keyCode in ch
     Fixed repeated typo in code that should never
        be reached anyway
       (had resetTimer instead of resetTimers)
     Replace hardcoded refresh rate of 3000 with 
        initialized variable to enable user customization
     Fixed bug where numeric keys reset search string
     Enabled shift-I for inner class summary
     Enabled search for classes starting with underscore
        by kludge to jump to end of list
*/

// You can change the following line to customize the time
// interval within which keystrokes belong to same class name
// (in milliseconds, so 1000 = 1 second)

var keyTimeout = 1000;

// The following is the refresh rate at which the entire script is
//   reinitialized.  This is necessary because a location change
//   in a frame clears the onUnload handler for that frame, so that
//   approach can't be used to refresh keystroke sensing in the frame.

var refreshRate = 3000;

var CR = 13, ESC = 27;

var start = 0, count = 0, keys = "", timer = null, reset = null,
    classLoc, pflinks;
var link = null, path = null, name = null;
var isIE = false, isN4 = false, isN6 = false;

function init() {
  if (document.all) isIE = true;
  else if (document.layers) isN4 = true;
  else if (document.getElementById) isN6 = true;
  classLoc = classFrame.location;
  packageFrame.focus();
  init2();
  }
function init2() {
  pflinks = packageFrame.document.links;
  if (isIE || isN4) {
    packageListFrame.document.onkeydown = goTo;
    packageFrame.document.onkeydown = goTo;
    classFrame.document.onkeydown = goTo;
    }
  else if (isN6) {
    packageListFrame.document.addEventListener('keydown', goTo, true);
    packageFrame.document.addEventListener('keydown', goTo, true);
    classFrame.document.addEventListener('keydown', goTo, true);
    }
  if (reset == null) reset = setInterval("init2()", refreshRate);
  }
function goTo(e) {
  var fr = null;
  if (timer != null) { clearTimeout(timer); timer = null; }
  if (reset != null) { clearInterval(reset); reset = null; }
  count++;
  if (isIE) {
    if (packageListFrame.event) fr = packageListFrame;
    else if (packageFrame.event) fr = packageFrame;
    else if (classFrame.event) fr = classFrame;
    else return resetTimers();
    e = fr.event;
    ch = e.keyCode;
    if (ch == CR) {
      classLoc.href = link.href;
      resetTimers();
      return;
      }
    var key;
    if (ch == 189) {
      if (keys == "") {
        packageFrame.scrollTo(0, pflinks[pflinks.length-1].offsetTop - 100);
        return resetTimers();
        }
      key = "_";
      }
    else {
      key = String.fromCharCode(ch).toLowerCase();
      if ( (  !(key >= "a" && key<= "z") 
             && !(key >="0" && key <= "9") 
             && key != "$")
           || e.altKey || e.ctrlKey) 
        return resetTimers();
      if (e.shiftKey) {
        if (key == "f") classLoc.hash = "field_summary";
        else if (key == "c") classLoc.hash = "constructor_summary";
        else if (key == "m") classLoc.hash = "method_summary";
        else if (key == "i") classLoc.hash = "inner_class_summary";
        return cancelGoTo();
        }
      }
    keys += key;
    }
  else if (isN4) {
    var keyCode = e.which, lcKey;
    if (keyCode == 0) return resetTimers();
    if (keyCode == CR) {
      classLoc.href = link.href;
      resetTimers();
      return;
      }
    key = String.fromCharCode(keyCode);

    if (key == "_") {
      if (keys == "") {
        packageFrame.scrollTo(0, packageFrame.document.height - 100);
        return resetTimers(); 
        }
      }
    lcKey = keyCode & ~32;
    if (e.modifiers == 4) {
      if (lcKey >= "a" && key1 <= "z") {
        if (lcKey == "f") classLoc.hash = "field_summary";
        else if (lcKey == "c") classLoc.hash = "constructor_summary";
        else if (lcKey == "m") classLoc.hash = "method_summary";
        else if (lcKey == "i") classLoc.hash = "inner_class_summary";
        return resetTimers();
        }
      if (keyCode != "_" && keyCode != "$") return resetTimers();
      }
    keys += lcKey;
    }
  else if (isN6) {
    ch = (e.keyCode) ? e.keyCode :
     (e.charCode) ? e.charCode : e.which;
    if (ch == CR) {
      classLoc.href = link.href;
      resetTimers();
      return;
      }
    else if (ch == ESC) ;
    else if (ch == 109) {
      if (keys == "") { 
        packageFrame.scrollTo(0, packageFrame.document.height - 100);
        return resetTimers();
        }
      }
    var key = String.fromCharCode(ch).toLowerCase();
    if (key < "a" || key > "z" || e.altKey || e.ctrlKey) 
      return resetTimers();
    if (e.shiftKey) {
      if (key == "f") classLoc.hash = "field_summary";
      else if (key == "c") classLoc.hash = "constructor_summary";
      else if (key == "m") classLoc.hash = "method_summary";
      else if (key == "i") classLoc.hash = "inner_class_summary";
      return cancelGoTo();
      }
    keys += key;

/**/    e.cancelBubble = true;
    }

  for (var i = start; i < pflinks.length; i++) { 
    link = pflinks[i];
    path = link.pathname;
    name = path.substr(Math.max(path.lastIndexOf("/"), path.lastIndexOf("\\")) + 1, count).toLowerCase();

    if (name >= keys) {
      if (isIE || isN6) packageFrame.scrollTo(0, link.offsetTop);
      else if (isN4) 
        packageFrame.scrollTo(0, 
            i * packageFrame.document.height / pflinks.length - 100);
      start = i;
      break;
      }
    }
  resetTimers();
  }
function resetTimers() {
  timer = setTimeout("cancelGoTo()", keyTimeout);
  }
function cancelGoTo() {
  start = 0;
  count = 0;
  keys = "";
  timer = null;
  init2();
  }
